@@ -7,10 +7,9 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: torrent-cell-renderer.c 11747 2011-01-22 13:26:44Z jordan $
+ * $Id: torrent-cell-renderer.c 12827 2011-09-01 22:49:35Z jordan $
  */
 
-#include <string.h> /* strcmp() */
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <libtransmission/transmission.h>
@@ -18,7 +17,6 @@
 #include "hig.h"
 #include "icons.h"
 #include "torrent-cell-renderer.h"
-#include "tr-torrent.h"
 #include "util.h"
 
 /* #define TEST_RTL */
@@ -41,8 +39,9 @@ enum
 ****
 ***/
 
-static char*
-getProgressString( const tr_torrent * tor,
+static void
+getProgressString( GString          * gstr,
+                   const tr_torrent * tor,
                    const tr_info    * info,
                    const tr_stat    * st )
 {
@@ -50,13 +49,12 @@ getProgressString( const tr_torrent * tor,
     const uint64_t haveTotal = st->haveUnchecked + st->haveValid;
     const int      isSeed = st->haveValid >= info->totalSize;
     char           buf1[32], buf2[32], buf3[32], buf4[32], buf5[32], buf6[32];
-    char *         str;
     double         seedRatio;
     const gboolean hasSeedRatio = tr_torrentGetSeedRatio( tor, &seedRatio );
 
     if( !isDone ) /* downloading */
     {
-        str = g_strdup_printf(
+        g_string_append_printf( gstr,
             /* %1$s is how much we've got,
                %2$s is how much we'll have when done,
                %3$s%% is a percentage of the two */
@@ -69,7 +67,7 @@ getProgressString( const tr_torrent * tor,
     {
         if( hasSeedRatio )
         {
-            str = g_strdup_printf(
+            g_string_append_printf( gstr,
                 /* %1$s is how much we've got,
                    %2$s is the torrent's total size,
                    %3$s%% is a percentage of the two,
@@ -86,7 +84,7 @@ getProgressString( const tr_torrent * tor,
         }
         else
         {
-            str = g_strdup_printf(
+            g_string_append_printf( gstr,
                 /* %1$s is how much we've got,
                    %2$s is the torrent's total size,
                    %3$s%% is a percentage of the two,
@@ -104,7 +102,7 @@ getProgressString( const tr_torrent * tor,
     {
         if( hasSeedRatio )
         {
-            str = g_strdup_printf(
+            g_string_append_printf( gstr,
                 /* %1$s is the torrent's total size,
                    %2$s is how much we've uploaded,
                    %3$s is our upload-to-download ratio,
@@ -117,7 +115,7 @@ getProgressString( const tr_torrent * tor,
         }
         else /* seeding w/o a ratio */
         {
-            str = g_strdup_printf(
+            g_string_append_printf( gstr,
                 /* %1$s is the torrent's total size,
                    %2$s is how much we've uploaded,
                    %3$s is our upload-to-download ratio */
@@ -133,7 +131,6 @@ getProgressString( const tr_torrent * tor,
         || ( hasSeedRatio && ( st->activity == TR_STATUS_SEED ) ) )
     {
         const int eta = st->eta;
-        GString * gstr = g_string_new( str );
         g_string_append( gstr, " - " );
         if( eta < 0 )
             g_string_append( gstr, _( "Remaining time unknown" ) );
@@ -144,11 +141,7 @@ getProgressString( const tr_torrent * tor,
             /* time remaining */
             g_string_append_printf( gstr, _( "%s remaining" ), timestr );
         }
-        g_free( str );
-        str = g_string_free( gstr, FALSE );
     }
-
-    return str;
 }
 
 static char*
@@ -182,8 +175,9 @@ getShortTransferString( const tr_torrent  * tor,
         /* bandwidth speed + unicode arrow */
         g_snprintf( buf, buflen, _( "%1$s %2$s" ),
                     gtr_get_unicode_string( GTR_UNICODE_UP ), upStr );
+    else if( st->isStalled )
+        g_strlcpy( buf, _( "Stalled" ), buflen );
     else if( haveMeta )
-        /* the torrent isn't uploading or downloading */
         g_strlcpy( buf, _( "Idle" ), buflen );
     else
         *buf = '\0';
@@ -191,30 +185,30 @@ getShortTransferString( const tr_torrent  * tor,
     return buf;
 }
 
-static char*
-getShortStatusString( const tr_torrent  * tor,
+static void
+getShortStatusString( GString           * gstr,
+                      const tr_torrent  * tor,
                       const tr_stat     * st,
                       double              uploadSpeed_KBps,
                       double              downloadSpeed_KBps )
 {
-    GString * gstr = g_string_new( NULL );
-
     switch( st->activity )
     {
         case TR_STATUS_STOPPED:
-            if( st->finished )
-                g_string_assign( gstr, _( "Finished" ) );
-            else
-                g_string_assign( gstr, _( "Paused" ) );
+            g_string_append( gstr, st->finished ? _( "Finished" ) : _( "Paused" ) );
             break;
-
         case TR_STATUS_CHECK_WAIT:
-            g_string_assign( gstr, _( "Waiting to verify local data" ) );
+            g_string_append( gstr, _( "Queued for verification" ) );
+            break;
+        case TR_STATUS_DOWNLOAD_WAIT:
+            g_string_append( gstr, _( "Queued for download" ) );
+            break;
+        case TR_STATUS_SEED_WAIT:
+            g_string_append( gstr, _( "Queued for seeding" ) );
             break;
 
         case TR_STATUS_CHECK:
-            g_string_append_printf( gstr,
-                                    _( "Verifying local data (%.1f%% tested)" ),
+            g_string_append_printf( gstr, _( "Verifying local data (%.1f%% tested)" ),
                                     tr_truncd( st->recheckProgress * 100.0, 1 ) );
             break;
 
@@ -236,22 +230,15 @@ getShortStatusString( const tr_torrent  * tor,
         default:
             break;
     }
-
-    return g_string_free( gstr, FALSE );
 }
 
-static char*
-getStatusString( const tr_torrent  * tor,
+static void
+getStatusString( GString           * gstr,
+                 const tr_torrent  * tor,
                  const tr_stat     * st,
                  const double        uploadSpeed_KBps,
                  const double        downloadSpeed_KBps )
 {
-    const int isActive = st->activity != TR_STATUS_STOPPED;
-    const int isChecking = st->activity == TR_STATUS_CHECK
-                        || st->activity == TR_STATUS_CHECK_WAIT;
-
-    GString * gstr = g_string_new( NULL );
-
     if( st->error )
     {
         const char * fmt[] = { NULL, N_( "Tracker gave a warning: \"%s\"" ),
@@ -264,10 +251,10 @@ getStatusString( const tr_torrent  * tor,
         case TR_STATUS_STOPPED:
         case TR_STATUS_CHECK_WAIT:
         case TR_STATUS_CHECK:
+        case TR_STATUS_DOWNLOAD_WAIT:
+        case TR_STATUS_SEED_WAIT:
         {
-            char * pch = getShortStatusString( tor, st, uploadSpeed_KBps, downloadSpeed_KBps );
-            g_string_assign( gstr, pch );
-            g_free( pch );
+            getShortStatusString( gstr, tor, st, uploadSpeed_KBps, downloadSpeed_KBps );
             break;
         }
 
@@ -276,18 +263,18 @@ getStatusString( const tr_torrent  * tor,
             if( tr_torrentHasMetadata( tor ) )
             {
                 g_string_append_printf( gstr,
-                    gtr_ngettext( "Downloading from %1$'d of %2$'d connected peer",
-                                  "Downloading from %1$'d of %2$'d connected peers",
-                                  st->webseedsSendingToUs + st->peersSendingToUs ),
+                    ngettext( "Downloading from %1$'d of %2$'d connected peer",
+                              "Downloading from %1$'d of %2$'d connected peers",
+                              st->webseedsSendingToUs + st->peersSendingToUs ),
                     st->webseedsSendingToUs + st->peersSendingToUs,
                     st->webseedsSendingToUs + st->peersConnected );
             }
             else
             {
                 g_string_append_printf( gstr,
-                    gtr_ngettext( "Downloading metadata from %1$'d peer (%2$d%% done)",
-                                  "Downloading metadata from %1$'d peers (%2$d%% done)",
-                                  st->peersConnected + st->peersConnected ),
+                    ngettext( "Downloading metadata from %1$'d peer (%2$d%% done)",
+                              "Downloading metadata from %1$'d peers (%2$d%% done)",
+                              st->peersConnected + st->peersConnected ),
                     st->peersConnected + st->webseedsSendingToUs,
                     (int)(100.0*st->metadataPercentComplete) );
             }
@@ -296,37 +283,39 @@ getStatusString( const tr_torrent  * tor,
 
         case TR_STATUS_SEED:
             g_string_append_printf( gstr,
-                gtr_ngettext( "Seeding to %1$'d of %2$'d connected peer",
-                              "Seeding to %1$'d of %2$'d connected peers",
-                              st->peersConnected ),
+                ngettext( "Seeding to %1$'d of %2$'d connected peer",
+                          "Seeding to %1$'d of %2$'d connected peers",
+                          st->peersConnected ),
                 st->peersGettingFromUs,
                 st->peersConnected );
                 break;
     }
 
-    if( isActive && !isChecking )
+    if( ( st->activity != TR_STATUS_CHECK_WAIT ) &&
+        ( st->activity != TR_STATUS_CHECK ) &&
+        ( st->activity != TR_STATUS_DOWNLOAD_WAIT ) &&
+        ( st->activity != TR_STATUS_SEED_WAIT ) &&
+        ( st->activity != TR_STATUS_STOPPED ) )
     {
         char buf[256];
         getShortTransferString( tor, st, uploadSpeed_KBps, downloadSpeed_KBps, buf, sizeof( buf ) );
         if( *buf )
             g_string_append_printf( gstr, " - %s", buf );
     }
-
-    return g_string_free( gstr, FALSE );
 }
 
 /***
 ****
 ***/
 
-static GtkCellRendererClass * parent_class = NULL;
-
 struct TorrentCellRendererPrivate
 {
     tr_torrent       * tor;
     GtkCellRenderer  * text_renderer;
     GtkCellRenderer  * progress_renderer;
     GtkCellRenderer  * icon_renderer;
+    GString          * gstr1;
+    GString          * gstr2;
     int bar_height;
 
     /* Use this instead of tr_stat.pieceUploadSpeed so that the model can
@@ -367,43 +356,57 @@ get_icon( const tr_torrent * tor, GtkIconSize icon_size, GtkWidget * for_widget
 ****
 ***/
 
+static void
+gtr_cell_renderer_get_preferred_size( GtkCellRenderer  * renderer,
+                                      GtkWidget        * widget,
+                                      GtkRequisition   * minimum_size,
+                                      GtkRequisition   * natural_size )
+{
+#if GTK_CHECK_VERSION( 3,0,0 )
+
+    gtk_cell_renderer_get_preferred_size( renderer, widget, minimum_size, natural_size );
+
+#else
+
+    GtkRequisition r;
+    gtk_cell_renderer_get_size( renderer, widget, NULL, NULL, NULL, &r.width, &r.height );
+    if( minimum_size ) *minimum_size = r;
+    if( natural_size ) *natural_size = r;
+
+#endif
+}
+
 static void
 get_size_compact( TorrentCellRenderer * cell,
                   GtkWidget           * widget,
                   gint                * width,
                   gint                * height )
 {
-    int w, h;
     int xpad, ypad;
-    GdkRectangle icon_area;
-    GdkRectangle name_area;
-    GdkRectangle stat_area;
+    GtkRequisition icon_size;
+    GtkRequisition name_size;
+    GtkRequisition stat_size;
     const char * name;
-    char * status;
     GdkPixbuf * icon;
 
     struct TorrentCellRendererPrivate * p = cell->priv;
     const tr_torrent * tor = p->tor;
     const tr_stat * st = tr_torrentStatCached( (tr_torrent*)tor );
+    GString * gstr_stat = p->gstr1;
 
     icon = get_icon( tor, COMPACT_ICON_SIZE, widget );
-    name = tr_torrentInfo( tor )->name;
-    status = getShortStatusString( tor, st, p->upload_speed_KBps, p->download_speed_KBps );
-    gtr_cell_renderer_get_padding( GTK_CELL_RENDERER( cell ), &xpad, &ypad );
+    name = tr_torrentName( tor );
+    g_string_truncate( gstr_stat, 0 );
+    getShortStatusString( gstr_stat, tor, st, p->upload_speed_KBps, p->download_speed_KBps );
+    gtk_cell_renderer_get_padding( GTK_CELL_RENDERER( cell ), &xpad, &ypad );
 
     /* get the idealized cell dimensions */
     g_object_set( p->icon_renderer, "pixbuf", icon, NULL );
-    gtk_cell_renderer_get_size( p->icon_renderer, widget, NULL, NULL, NULL, &w, &h );
-    icon_area.width = w;
-    icon_area.height = h;
+    gtr_cell_renderer_get_preferred_size( p->icon_renderer, widget, NULL, &icon_size );
     g_object_set( p->text_renderer, "text", name, "ellipsize", PANGO_ELLIPSIZE_NONE,  "scale", 1.0, NULL );
-    gtk_cell_renderer_get_size( p->text_renderer, widget, NULL, NULL, NULL, &w, &h );
-    name_area.width = w;
-    name_area.height = h;
-    g_object_set( p->text_renderer, "text", status, "scale", SMALL_SCALE, NULL );
-    gtk_cell_renderer_get_size( p->text_renderer, widget, NULL, NULL, NULL, &w, &h );
-    stat_area.width = w;
-    stat_area.height = h;
+    gtr_cell_renderer_get_preferred_size( p->text_renderer, widget, NULL, &name_size );
+    g_object_set( p->text_renderer, "text", gstr_stat->str, "scale", SMALL_SCALE, NULL );
+    gtr_cell_renderer_get_preferred_size( p->text_renderer, widget, NULL, &stat_size );
 
     /**
     *** LAYOUT
@@ -411,12 +414,11 @@ get_size_compact( TorrentCellRenderer * cell,
 
 #define BAR_WIDTH 50
     if( width != NULL )
-        *width = xpad * 2 + icon_area.width + GUI_PAD + name_area.width + GUI_PAD + BAR_WIDTH + GUI_PAD + stat_area.width;
+        *width = xpad * 2 + icon_size.width + GUI_PAD + name_size.width + GUI_PAD + BAR_WIDTH + GUI_PAD + stat_size.width;
     if( height != NULL )
-        *height = ypad * 2 + MAX( name_area.height, p->bar_height );
+        *height = ypad * 2 + MAX( name_size.height, p->bar_height );
 
     /* cleanup */
-    g_free( status );
     g_object_unref( icon );
 }
 
@@ -428,70 +430,65 @@ get_size_full( TorrentCellRenderer * cell,
                gint                * width,
                gint                * height )
 {
-    int w, h;
     int xpad, ypad;
-    GdkRectangle icon_area;
-    GdkRectangle name_area;
-    GdkRectangle stat_area;
-    GdkRectangle prog_area;
+    GtkRequisition icon_size;
+    GtkRequisition name_size;
+    GtkRequisition stat_size;
+    GtkRequisition prog_size;
     const char * name;
-    char * status;
-    char * progress;
     GdkPixbuf * icon;
 
     struct TorrentCellRendererPrivate * p = cell->priv;
     const tr_torrent * tor = p->tor;
     const tr_stat * st = tr_torrentStatCached( (tr_torrent*)tor );
     const tr_info * inf = tr_torrentInfo( tor );
+    GString * gstr_prog = p->gstr1;
+    GString * gstr_stat = p->gstr2;
 
     icon = get_icon( tor, FULL_ICON_SIZE, widget );
-    name = inf->name;
-    status = getStatusString( tor, st, p->upload_speed_KBps, p->download_speed_KBps );
-    progress = getProgressString( tor, inf, st );
-    gtr_cell_renderer_get_padding( GTK_CELL_RENDERER( cell ), &xpad, &ypad );
+    name = tr_torrentName( tor );
+    g_string_truncate( gstr_stat, 0 );
+    getStatusString( gstr_stat, tor, st, p->upload_speed_KBps, p->download_speed_KBps );
+    g_string_truncate( gstr_prog, 0 );
+    getProgressString( gstr_prog, tor, inf, st );
+    gtk_cell_renderer_get_padding( GTK_CELL_RENDERER( cell ), &xpad, &ypad );
 
     /* get the idealized cell dimensions */
     g_object_set( p->icon_renderer, "pixbuf", icon, NULL );
-    gtk_cell_renderer_get_size( p->icon_renderer, widget, NULL, NULL, NULL, &w, &h );
-    icon_area.width = w;
-    icon_area.height = h;
+    gtr_cell_renderer_get_preferred_size( p->icon_renderer, widget, NULL, &icon_size );
     g_object_set( p->text_renderer, "text", name, "weight", PANGO_WEIGHT_BOLD, "scale", 1.0, "ellipsize", PANGO_ELLIPSIZE_NONE, NULL );
-    gtk_cell_renderer_get_size( p->text_renderer, widget, NULL, NULL, NULL, &w, &h );
-    name_area.width = w;
-    name_area.height = h;
-    g_object_set( p->text_renderer, "text", progress, "weight", PANGO_WEIGHT_NORMAL, "scale", SMALL_SCALE, NULL );
-    gtk_cell_renderer_get_size( p->text_renderer, widget, NULL, NULL, NULL, &w, &h );
-    prog_area.width = w;
-    prog_area.height = h;
-    g_object_set( p->text_renderer, "text", status, NULL );
-    gtk_cell_renderer_get_size( p->text_renderer, widget, NULL, NULL, NULL, &w, &h );
-    stat_area.width = w;
-    stat_area.height = h;
+    gtr_cell_renderer_get_preferred_size( p->text_renderer, widget, NULL, &name_size );
+    g_object_set( p->text_renderer, "text", gstr_prog->str, "weight", PANGO_WEIGHT_NORMAL, "scale", SMALL_SCALE, NULL );
+    gtr_cell_renderer_get_preferred_size( p->text_renderer, widget, NULL, &prog_size );
+    g_object_set( p->text_renderer, "text", gstr_stat->str, NULL );
+    gtr_cell_renderer_get_preferred_size( p->text_renderer, widget, NULL, &stat_size );
 
     /**
     *** LAYOUT
     **/
 
     if( width != NULL )
-        *width = xpad * 2 + icon_area.width + GUI_PAD + MAX3( name_area.width, prog_area.width, stat_area.width );
+        *width = xpad * 2 + icon_size.width + GUI_PAD + MAX3( name_size.width, prog_size.width, stat_size.width );
     if( height != NULL )
-        *height = ypad * 2 + name_area.height + prog_area.height + GUI_PAD_SMALL + p->bar_height + GUI_PAD_SMALL + stat_area.height;
+        *height = ypad * 2 + name_size.height + prog_size.height + GUI_PAD_SMALL + p->bar_height + GUI_PAD_SMALL + stat_size.height;
 
     /* cleanup */
-    g_free( status );
-    g_free( progress );
     g_object_unref( icon );
 }
 
 
 static void
-torrent_cell_renderer_get_size( GtkCellRenderer  * cell,
-                                GtkWidget        * widget,
-                                GdkRectangle     * cell_area,
-                                gint             * x_offset,
-                                gint             * y_offset,
-                                gint             * width,
-                                gint             * height )
+torrent_cell_renderer_get_size( GtkCellRenderer     * cell,
+                                GtkWidget           * widget,
+#if GTK_CHECK_VERSION( 3,0,0 )
+                                const GdkRectangle  * cell_area,
+#else
+                                GdkRectangle        * cell_area,
+#endif
+                                gint                * x_offset,
+                                gint                * y_offset,
+                                gint                * width,
+                                gint                * height )
 {
     TorrentCellRenderer * self = TORRENT_CELL_RENDERER( cell );
 
@@ -516,56 +513,119 @@ torrent_cell_renderer_get_size( GtkCellRenderer  * cell,
 
         if( y_offset ) {
             int xpad, ypad;
-            gtr_cell_renderer_get_padding( cell, &xpad, &ypad );
+            gtk_cell_renderer_get_padding( cell, &xpad, &ypad );
             *y_offset = cell_area ? (int)((cell_area->height - (ypad*2 +h)) / 2.0) : 0;
         }
     }
 }
 
-static void
-get_text_color( GtkWidget * w, const tr_stat * st, GdkColor * setme )
-{
-    static const GdkColor red = { 0, 65535, 0, 0 };
+#if GTK_CHECK_VERSION( 3,0,0 )
+    typedef GdkRGBA GtrColor;
+    #define FOREGROUND_COLOR_KEY "foreground-rgba"
+#else
+    typedef GdkColor GtrColor;
+    #define FOREGROUND_COLOR_KEY "foreground-gdk"
+#endif
 
+static void
+get_text_color( GtkWidget * w, const tr_stat * st, GtrColor * setme )
+{
+#if GTK_CHECK_VERSION( 3,0,0 )
+
+    static const GdkRGBA red = { 1.0, 0, 0, 0 };
+    if( st->error )
+        *setme = red;
+    else if( st->activity == TR_STATUS_STOPPED )
+        gtk_style_context_get_color( gtk_widget_get_style_context( w ), GTK_STATE_FLAG_INSENSITIVE, setme );
+    else
+        gtk_style_context_get_color( gtk_widget_get_style_context( w ), GTK_STATE_FLAG_NORMAL, setme );
+
+#else
+
+    static const GdkColor red = { 0, 65535, 0, 0 };
     if( st->error )
         *setme = red;
     else if( st->activity == TR_STATUS_STOPPED )
         *setme = gtk_widget_get_style(w)->text[GTK_STATE_INSENSITIVE];
     else
         *setme = gtk_widget_get_style(w)->text[GTK_STATE_NORMAL];
+
+#endif
+}
+
+
+static double
+get_percent_done( const tr_torrent * tor, const tr_stat * st, bool * seed )
+{
+    double d;
+
+    if( ( st->activity == TR_STATUS_SEED ) && tr_torrentGetSeedRatio( tor, &d ) )
+    {
+        *seed = true;
+        d = MAX( 0.0, st->seedRatioPercentDone );
+    }
+    else
+    {
+        *seed = false;
+        d = MAX( 0.0, st->percentDone );
+    }
+
+    return d;
+}
+
+#if GTK_CHECK_VERSION( 3,0,0 )
+    typedef cairo_t GtrDrawable;
+#else
+    typedef GdkDrawable GtrDrawable;
+#endif
+
+static void
+gtr_cell_renderer_render( GtkCellRenderer       * renderer,
+                          GtrDrawable           * drawable,
+                          GtkWidget             * widget,
+                          const GdkRectangle    * area,
+                          GtkCellRendererState    flags)
+{
+#if GTK_CHECK_VERSION( 3,0,0 )
+    gtk_cell_renderer_render( renderer, drawable, widget, area, area, flags );
+#else
+    gtk_cell_renderer_render( renderer, drawable, widget, area, area, area, flags );
+#endif
 }
 
 static void
 render_compact( TorrentCellRenderer   * cell,
-                GdkDrawable           * window,
+                GtrDrawable           * window,
                 GtkWidget             * widget,
-                GdkRectangle          * background_area,
-                GdkRectangle          * cell_area UNUSED,
-                GdkRectangle          * expose_area UNUSED,
+                const GdkRectangle    * background_area,
+                const GdkRectangle    * cell_area UNUSED,
                 GtkCellRendererState    flags )
 {
     int xpad, ypad;
+    GtkRequisition size;
     GdkRectangle icon_area;
     GdkRectangle name_area;
     GdkRectangle stat_area;
     GdkRectangle prog_area;
     GdkRectangle fill_area;
     const char * name;
-    char * status;
     GdkPixbuf * icon;
-    GdkColor text_color;
+    GtrColor text_color;
+    bool seed;
 
     struct TorrentCellRendererPrivate * p = cell->priv;
     const tr_torrent * tor = p->tor;
     const tr_stat * st = tr_torrentStatCached( (tr_torrent*)tor );
-    const gboolean active = st->activity != TR_STATUS_STOPPED;
-    const double percentDone = MAX( 0.0, st->percentDone );
+    const gboolean active = ( st->activity != TR_STATUS_STOPPED ) && ( st->activity != TR_STATUS_DOWNLOAD_WAIT ) && ( st->activity != TR_STATUS_SEED_WAIT );
+    const double percentDone = get_percent_done( tor, st, &seed );
     const gboolean sensitive = active || st->error;
+    GString * gstr_stat = p->gstr1;
 
     icon = get_icon( tor, COMPACT_ICON_SIZE, widget );
-    name = tr_torrentInfo( tor )->name;
-    status = getShortStatusString( tor, st, p->upload_speed_KBps, p->download_speed_KBps );
-    gtr_cell_renderer_get_padding( GTK_CELL_RENDERER( cell ), &xpad, &ypad );
+    name = tr_torrentName( tor );
+    g_string_truncate( gstr_stat, 0 );
+    getShortStatusString( gstr_stat, tor, st, p->upload_speed_KBps, p->download_speed_KBps );
+    gtk_cell_renderer_get_padding( GTK_CELL_RENDERER( cell ), &xpad, &ypad );
     get_text_color( widget, st, &text_color );
 
     fill_area = *background_area;
@@ -576,11 +636,14 @@ render_compact( TorrentCellRenderer   * cell,
     icon_area = name_area = stat_area = prog_area = fill_area;
 
     g_object_set( p->icon_renderer, "pixbuf", icon, NULL );
-    gtk_cell_renderer_get_size( p->icon_renderer, widget, NULL, NULL, NULL, &icon_area.width, NULL );
+    gtr_cell_renderer_get_preferred_size( p->icon_renderer, widget, NULL, &size );
+    icon_area.width = size.width;
     g_object_set( p->text_renderer, "text", name, "ellipsize", PANGO_ELLIPSIZE_NONE, "scale", 1.0, NULL );
-    gtk_cell_renderer_get_size( p->text_renderer, widget, NULL, NULL, NULL, &name_area.width, NULL );
-    g_object_set( p->text_renderer, "text", status, "scale", SMALL_SCALE, NULL );
-    gtk_cell_renderer_get_size( p->text_renderer, widget, NULL, NULL, NULL, &stat_area.width, NULL );
+    gtr_cell_renderer_get_preferred_size( p->text_renderer, widget, NULL, &size );
+    name_area.width = size.width;
+    g_object_set( p->text_renderer, "text", gstr_stat->str, "scale", SMALL_SCALE, NULL );
+    gtr_cell_renderer_get_preferred_size( p->text_renderer, widget, NULL, &size );
+    stat_area.width = size.width;
 
     icon_area.x = fill_area.x;
     prog_area.x = fill_area.x + fill_area.width - BAR_WIDTH;
@@ -595,30 +658,28 @@ render_compact( TorrentCellRenderer   * cell,
     **/
 
     g_object_set( p->icon_renderer, "pixbuf", icon, "sensitive", sensitive, NULL );
-    gtk_cell_renderer_render( p->icon_renderer, window, widget, &icon_area, &icon_area, &icon_area, flags );
+    gtr_cell_renderer_render( p->icon_renderer, window, widget, &icon_area, flags );
     g_object_set( p->progress_renderer, "value", (int)(percentDone*100.0), "text", NULL, "sensitive", sensitive, NULL );
-    gtk_cell_renderer_render( p->progress_renderer, window, widget, &prog_area, &prog_area, &prog_area, flags );
-    g_object_set( p->text_renderer, "text", status, "scale", SMALL_SCALE, "ellipsize", PANGO_ELLIPSIZE_END, "foreground-gdk", &text_color, NULL );
-    gtk_cell_renderer_render( p->text_renderer, window, widget, &stat_area, &stat_area, &stat_area, flags );
-    g_object_set( p->text_renderer, "text", name, "scale", 1.0, NULL );
-    gtk_cell_renderer_render( p->text_renderer, window, widget, &name_area, &name_area, &name_area, flags );
+    gtr_cell_renderer_render( p->progress_renderer, window, widget, &prog_area, flags );
+    g_object_set( p->text_renderer, "text", gstr_stat->str, "scale", SMALL_SCALE, "ellipsize", PANGO_ELLIPSIZE_END, FOREGROUND_COLOR_KEY, &text_color, NULL );
+    gtr_cell_renderer_render( p->text_renderer, window, widget, &stat_area, flags );
+    g_object_set( p->text_renderer, "text", name, "scale", 1.0, FOREGROUND_COLOR_KEY, &text_color, NULL );
+    gtr_cell_renderer_render( p->text_renderer, window, widget, &name_area, flags );
 
     /* cleanup */
-    g_free( status );
     g_object_unref( icon );
 }
 
 static void
 render_full( TorrentCellRenderer   * cell,
-             GdkDrawable           * window,
+             GtrDrawable           * window,
              GtkWidget             * widget,
-             GdkRectangle          * background_area,
-             GdkRectangle          * cell_area UNUSED,
-             GdkRectangle          * expose_area UNUSED,
+             const GdkRectangle    * background_area,
+             const GdkRectangle    * cell_area UNUSED,
              GtkCellRendererState    flags )
 {
-    int w, h;
     int xpad, ypad;
+    GtkRequisition size;
     GdkRectangle fill_area;
     GdkRectangle icon_area;
     GdkRectangle name_area;
@@ -626,43 +687,46 @@ render_full( TorrentCellRenderer   * cell,
     GdkRectangle prog_area;
     GdkRectangle prct_area;
     const char * name;
-    char * status;
-    char * progress;
     GdkPixbuf * icon;
-    GdkColor text_color;
+    GtrColor text_color;
+    bool seed;
 
     struct TorrentCellRendererPrivate * p = cell->priv;
     const tr_torrent * tor = p->tor;
     const tr_stat * st = tr_torrentStatCached( (tr_torrent*)tor );
     const tr_info * inf = tr_torrentInfo( tor );
-    const gboolean active = st->activity != TR_STATUS_STOPPED;
-    const double percentDone = MAX( 0.0, st->percentDone );
+    const gboolean active = ( st->activity != TR_STATUS_STOPPED ) && ( st->activity != TR_STATUS_DOWNLOAD_WAIT ) && ( st->activity != TR_STATUS_SEED_WAIT );
+    const double percentDone = get_percent_done( tor, st, &seed );
     const gboolean sensitive = active || st->error;
+    GString * gstr_prog = p->gstr1;
+    GString * gstr_stat = p->gstr2;
 
     icon = get_icon( tor, FULL_ICON_SIZE, widget );
-    name = inf->name;
-    status = getStatusString( tor, st, p->upload_speed_KBps, p->download_speed_KBps );
-    progress = getProgressString( tor, inf, st );
-    gtr_cell_renderer_get_padding( GTK_CELL_RENDERER( cell ), &xpad, &ypad );
+    name = tr_torrentName( tor );
+    g_string_truncate( gstr_prog, 0 );
+    getProgressString( gstr_prog, tor, inf, st );
+    g_string_truncate( gstr_stat, 0 );
+    getStatusString( gstr_stat, tor, st, p->upload_speed_KBps, p->download_speed_KBps );
+    gtk_cell_renderer_get_padding( GTK_CELL_RENDERER( cell ), &xpad, &ypad );
     get_text_color( widget, st, &text_color );
 
     /* get the idealized cell dimensions */
     g_object_set( p->icon_renderer, "pixbuf", icon, NULL );
-    gtk_cell_renderer_get_size( p->icon_renderer, widget, NULL, NULL, NULL, &w, &h );
-    icon_area.width = w;
-    icon_area.height = h;
+    gtr_cell_renderer_get_preferred_size( p->icon_renderer, widget, NULL, &size );
+    icon_area.width = size.width;
+    icon_area.height = size.height;
     g_object_set( p->text_renderer, "text", name, "weight", PANGO_WEIGHT_BOLD, "ellipsize", PANGO_ELLIPSIZE_NONE, "scale", 1.0, NULL );
-    gtk_cell_renderer_get_size( p->text_renderer, widget, NULL, NULL, NULL, &w, &h );
-    name_area.width = w;
-    name_area.height = h;
-    g_object_set( p->text_renderer, "text", progress, "weight", PANGO_WEIGHT_NORMAL, "scale", SMALL_SCALE, NULL );
-    gtk_cell_renderer_get_size( p->text_renderer, widget, NULL, NULL, NULL, &w, &h );
-    prog_area.width = w;
-    prog_area.height = h;
-    g_object_set( p->text_renderer, "text", status, NULL );
-    gtk_cell_renderer_get_size( p->text_renderer, widget, NULL, NULL, NULL, &w, &h );
-    stat_area.width = w;
-    stat_area.height = h;
+    gtr_cell_renderer_get_preferred_size( p->text_renderer, widget, NULL, &size );
+    name_area.width = size.width;
+    name_area.height = size.height;
+    g_object_set( p->text_renderer, "text", gstr_prog->str, "weight", PANGO_WEIGHT_NORMAL, "scale", SMALL_SCALE, NULL );
+    gtr_cell_renderer_get_preferred_size( p->text_renderer, widget, NULL, &size );
+    prog_area.width = size.width;
+    prog_area.height = size.height;
+    g_object_set( p->text_renderer, "text", gstr_stat->str, NULL );
+    gtr_cell_renderer_get_preferred_size( p->text_renderer, widget, NULL, &size );
+    stat_area.width = size.width;
+    stat_area.height = size.height;
 
     /**
     *** LAYOUT
@@ -704,29 +768,32 @@ render_full( TorrentCellRenderer   * cell,
     **/
 
     g_object_set( p->icon_renderer, "pixbuf", icon, "sensitive", sensitive, NULL );
-    gtk_cell_renderer_render( p->icon_renderer, window, widget, &icon_area, &icon_area, &icon_area, flags );
-    g_object_set( p->text_renderer, "text", name, "scale", 1.0, "foreground-gdk", &text_color, "ellipsize", PANGO_ELLIPSIZE_END, "weight", PANGO_WEIGHT_BOLD, NULL );
-    gtk_cell_renderer_render( p->text_renderer, window, widget, &name_area, &name_area, &name_area, flags );
-    g_object_set( p->text_renderer, "text", progress, "scale", SMALL_SCALE, "weight", PANGO_WEIGHT_NORMAL, NULL );
-    gtk_cell_renderer_render( p->text_renderer, window, widget, &prog_area, &prog_area, &prog_area, flags );
+    gtr_cell_renderer_render( p->icon_renderer, window, widget, &icon_area, flags );
+    g_object_set( p->text_renderer, "text", name, "scale", 1.0, FOREGROUND_COLOR_KEY, &text_color, "ellipsize", PANGO_ELLIPSIZE_END, "weight", PANGO_WEIGHT_BOLD, NULL );
+    gtr_cell_renderer_render( p->text_renderer, window, widget, &name_area, flags );
+    g_object_set( p->text_renderer, "text", gstr_prog->str, "scale", SMALL_SCALE, "weight", PANGO_WEIGHT_NORMAL, NULL );
+    gtr_cell_renderer_render( p->text_renderer, window, widget, &prog_area, flags );
     g_object_set( p->progress_renderer, "value", (int)(percentDone*100.0), "text", "", "sensitive", sensitive, NULL );
-    gtk_cell_renderer_render( p->progress_renderer, window, widget, &prct_area, &prct_area, &prct_area, flags );
-    g_object_set( p->text_renderer, "text", status, NULL );
-    gtk_cell_renderer_render( p->text_renderer, window, widget, &stat_area, &stat_area, &stat_area, flags );
+    gtr_cell_renderer_render( p->progress_renderer, window, widget, &prct_area, flags );
+    g_object_set( p->text_renderer, "text", gstr_stat->str, FOREGROUND_COLOR_KEY, &text_color, NULL );
+    gtr_cell_renderer_render( p->text_renderer, window, widget, &stat_area, flags );
 
     /* cleanup */
-    g_free( status );
-    g_free( progress );
     g_object_unref( icon );
 }
 
 static void
 torrent_cell_renderer_render( GtkCellRenderer       * cell,
-                              GdkDrawable           * window,
+                              GtrDrawable           * window,
                               GtkWidget             * widget,
+#if GTK_CHECK_VERSION( 3,0,0 )
+                              const GdkRectangle    * background_area,
+                              const GdkRectangle    * cell_area,
+#else
                               GdkRectangle          * background_area,
                               GdkRectangle          * cell_area,
-                              GdkRectangle          * expose_area,
+                              GdkRectangle          * expose_area UNUSED,
+#endif
                               GtkCellRendererState    flags )
 {
     TorrentCellRenderer * self = TORRENT_CELL_RENDERER( cell );
@@ -740,9 +807,9 @@ torrent_cell_renderer_render( GtkCellRenderer       * cell,
     {
         struct TorrentCellRendererPrivate * p = self->priv;
         if( p->compact )
-            render_compact( self, window, widget, background_area, cell_area, expose_area, flags );
+            render_compact( self, window, widget, background_area, cell_area, flags );
         else
-            render_full( self, window, widget, background_area, cell_area, expose_area, flags );
+            render_full( self, window, widget, background_area, cell_area, flags );
     }
 
 #ifdef TEST_RTL
@@ -790,22 +857,24 @@ torrent_cell_renderer_get_property( GObject     * object,
     }
 }
 
+G_DEFINE_TYPE (TorrentCellRenderer, torrent_cell_renderer, GTK_TYPE_CELL_RENDERER)
+
 static void
 torrent_cell_renderer_dispose( GObject * o )
 {
     TorrentCellRenderer * r = TORRENT_CELL_RENDERER( o );
-    GObjectClass *        parent;
 
     if( r && r->priv )
     {
+        g_string_free( r->priv->gstr1, TRUE );
+        g_string_free( r->priv->gstr2, TRUE );
         g_object_unref( G_OBJECT( r->priv->text_renderer ) );
         g_object_unref( G_OBJECT( r->priv->progress_renderer ) );
         g_object_unref( G_OBJECT( r->priv->icon_renderer ) );
         r->priv = NULL;
     }
 
-    parent = g_type_class_peek( g_type_parent( TORRENT_CELL_RENDERER_TYPE ) );
-    parent->dispose( o );
+    G_OBJECT_CLASS( torrent_cell_renderer_parent_class )->dispose( o );
 }
 
 static void
@@ -817,8 +886,6 @@ torrent_cell_renderer_class_init( TorrentCellRendererClass * klass )
     g_type_class_add_private( klass,
                              sizeof( struct TorrentCellRendererPrivate ) );
 
-    parent_class = (GtkCellRendererClass*) g_type_class_peek_parent( klass );
-
     cell_class->render = torrent_cell_renderer_render;
     cell_class->get_size = torrent_cell_renderer_get_size;
     gobject_class->set_property = torrent_cell_renderer_set_property;
@@ -857,11 +924,8 @@ torrent_cell_renderer_class_init( TorrentCellRendererClass * klass )
 }
 
 static void
-torrent_cell_renderer_init( GTypeInstance *  instance,
-                            gpointer g_class UNUSED )
+torrent_cell_renderer_init( TorrentCellRenderer * self )
 {
-    TorrentCellRenderer *               self = TORRENT_CELL_RENDERER(
-        instance );
     struct TorrentCellRendererPrivate * p;
 
     p = self->priv = G_TYPE_INSTANCE_GET_PRIVATE(
@@ -871,46 +935,19 @@ torrent_cell_renderer_init( GTypeInstance *  instance,
             TorrentCellRendererPrivate );
 
     p->tor = NULL;
+    p->gstr1 = g_string_new( NULL );
+    p->gstr2 = g_string_new( NULL );
     p->text_renderer = gtk_cell_renderer_text_new( );
     g_object_set( p->text_renderer, "xpad", 0, "ypad", 0, NULL );
     p->progress_renderer = gtk_cell_renderer_progress_new(  );
     p->icon_renderer = gtk_cell_renderer_pixbuf_new(  );
-    gtr_object_ref_sink( p->text_renderer );
-    gtr_object_ref_sink( p->progress_renderer );
-    gtr_object_ref_sink( p->icon_renderer );
+    g_object_ref_sink( p->text_renderer );
+    g_object_ref_sink( p->progress_renderer );
+    g_object_ref_sink( p->icon_renderer );
 
     p->bar_height = DEFAULT_BAR_HEIGHT;
 }
 
-GType
-torrent_cell_renderer_get_type( void )
-{
-    static GType type = 0;
-
-    if( !type )
-    {
-        static const GTypeInfo info =
-        {
-            sizeof( TorrentCellRendererClass ),
-            NULL,                                            /* base_init */
-            NULL,                                            /* base_finalize */
-            (GClassInitFunc)torrent_cell_renderer_class_init,
-            NULL,                                            /* class_finalize
-                                                               */
-            NULL,                                            /* class_data */
-            sizeof( TorrentCellRenderer ),
-            0,                                               /* n_preallocs */
-            (GInstanceInitFunc)torrent_cell_renderer_init,
-            NULL
-        };
-
-        type = g_type_register_static( GTK_TYPE_CELL_RENDERER,
-                                       "TorrentCellRenderer",
-                                       &info, (GTypeFlags)0 );
-    }
-
-    return type;
-}
 
 GtkCellRenderer *
 torrent_cell_renderer_new( void )
